package aktbus.demo.applet;
//
// AktMessage receiving applet using thread
//
import aktbus.core.*;
import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class TestReceiveAppletWithThread extends Applet implements ActionListener
{
TextField port_field;			//field for entering port no.
Label port_label,message_label;		//labels
Button wait_button,clear_button;		//wait & clear buttons
TextArea message_area;			//text-area displaying message
private volatile ReceiverThread rt=null;	//the receiving thread


//
// init method of applet
//
public void init()
{
GridBagConstraints c;				//constraint layout object
GridBagLayout layout;				//layout manager of GUI objects

layout=new GridBagLayout();
this.setLayout(layout);
c=new GridBagConstraints();

port_field=new TextField(6);			//TextField for entering port no.
port_field.setEditable(true);			//editable
message_area=new TextArea(5,40);			//TextArea to display message
message_area.setEditable(false);			//display only

port_label=new Label("Socket Port No.:");		//labels
message_label=new Label("Message Received:");

wait_button=new Button("Wait for message");	//wait button
wait_button.addActionListener(this);		//set listener for "wait" button
wait_button.setActionCommand("wait");
clear_button=new Button("Clear");			//clear button
clear_button.addActionListener(this);		//set listener
clear_button.setActionCommand("clear");

put_component(layout,c,port_label,0,0);		//put components into applet subjects to layout constraints
put_component(layout,c,port_field,1,0);
put_component(layout,c,message_label,0,2);
put_component(layout,c,message_area,1,2);
put_component(layout,c,wait_button,0,3);
put_component(layout,c,clear_button,1,3);
}

//
//put a component into applet panel subjected to grid constraint
//
private void put_component(GridBagLayout layout,GridBagConstraints c,Component p,int x,int y)
{
c.gridx=x;
c.gridy=y;
layout.setConstraints(p,c);	//set constraint on component
add(p);				//add component into apple panel
}

//
//start method of applet
//
public void start()
{
port_field.setText("");		//clear port textfield
message_area.setText("");		//clear message area
if (rt!=null)			//if there is a thread running, stop it
	{
	rt.interrupt();
	rt=null;
	}
}

//
// method called when applet stops
//
public void stop()
{
if (rt!=null)
	{
	rt.interrupt();
	rt=null;		//tell receiver thread to stop
	}
}


//
//method for handling button actions
//
public void actionPerformed(ActionEvent e)
{
String c;

c=e.getActionCommand();			//get action command from event

if (c.equals("clear"))
	{				//"clear" button pressed
	clear_or_stop();
	}
else if (c.equals("wait"))
	{			//"wait" button pressed
	wait_for_message();
	}
}

//
// once "wait" button is pressed, wait for incoming message
//
public void wait_for_message()
{
if (rt==null)		//no thread yet
	{
	try	{
		rt=new ReceiverThread(Integer.parseInt(port_field.getText()),message_area);	//create thread
		rt.start();								//start thread
		clear_button.setLabel("Stop");			//if thread runs, then change clear button
		clear_button.repaint();				//to "stop"
		} catch (NumberFormatException e)
			{
			message_area.append("Please enter a port no. to use!!!\n");
			}
		catch (Exception e)
			{
			clear_button.setLabel("Clear");		//if exception happens
			clear_button.repaint();			//reset "stop" button to "clear"
			System.out.println(e.toString());
			}
	}
else	{
	message_area.append("already waiting...\n");
	}
}


//
//clear or stop button pressed
//
public void clear_or_stop()
{
if (rt!=null)
	{
	rt.interrupt();			//try to interrupt thread
	rt=null;				//set state telling thread to stop
	}
clear_button.setLabel("Clear");	//set clear button label
clear_button.repaint();
port_field.setText("");		//clear port textfield
message_area.setText("");		//clear message area
}
}


