package aktbus.kqml;
import com.sun.xml.tree.*;

/**
* A KQML parameter which has a name and a value.
* The name must be a <em>String</em> while the value can be a String,
* or an XML subtree.
*/
public class KqmlParam
{
/**
* The name of the KQML parameter in a <em>String</em>.
*/
public String name;
/**
* The value of the KQML parameter.
*/
public Object value;

/**
* Construct a <em>KqmlParam</em> with a <em>String</em> value.
*
* @param name	The name of the KQML parameter in a <em>String</em>.
* @param value	The value of the KQML parameter in a <em>String</em>.
*/
public KqmlParam(String name,String value)
{
this.name=name;
this.value=value;
}

/**
* Construct a <em>KqmlParam</em>, taking an XML document as its value.
*
* @param name	The name of the KQML parameter in a <em>String</em>.
* @param value	The value of the KQML parameter, which is a <em>com.sun.xml.tree.XmlDocument</em> XML document.
*/
public KqmlParam(String name,com.sun.xml.tree.XmlDocument value)
{
this.name=name;
this.value=value;
}

/**
* Construct a KqmlParam, taking a <em>org.w3c.org.dom.Node</em> (w/ its subtree underneath) as its value.
*
* @param name	The name of the KQML parameter in a <em>String</em>.
* @param value	The value of the KQML parameter in a <em>org.w3c.dom.Node</em>.
*/
public KqmlParam(String name,org.w3c.dom.Node value)
{
this.name=name;
this.value=value;
}


}

