//
// AKT-Bus Linda gateway interfacing AKT-Bus's HTTP/RDF to Linda
//

import java.net.*;
import java.io.*;
import Linja.*;
import com.sun.xml.tree.*;
//
// the AtkBusLindaGateway class
//
public class AktBusLindaGateway
{
private String linda_host;			//linda-server host
private int linda_port;				//linda-server port
private String my_name;				//gateway's kraft name
private int my_port;				//gateways' server socket port
private boolean verbose=false;			//debug message flag, default off

//
// the constructor
//
public AktBusLindaGateway(	String l_host,		//linda server host
				int l_port,		//linda server port
				String myname,		//my name
				int myport,		//my port no. to use
				boolean verbose)
	throws Exception
{
//
// init parameters
//
this.linda_host=l_host;
this.linda_port=l_port;
this.my_port=myport;
this.my_name=myname;
this.verbose=verbose;
}

//
// main gateway method
//
public void work() throws Exception
{
AktBusLindaOutGate out_gate;
AktBusLindaInGate in_gate;

//start the outgoing gate thread

out_gate=new AktBusLindaOutGate(linda_host,linda_port,my_name,my_port,verbose);
out_gate.start();

//start the incoming gate thread

in_gate=new AktBusLindaInGate(linda_host,linda_port,my_name,my_port,verbose);
in_gate.start();
}
} //end AktBusLindaInGate




//
//the incoming-gate class
//

class AktBusLindaInGate extends Thread
{
LindaClient client;		//linda-client
String myname;			//gateway's kraft name
String mysite;			//my local site name
int myport;			//the server socket port
AktBusReceiver receiver;	//the AktBusReceiver object that receives an incoming AktMessage
boolean verbose;		//debug message flag

//
// constructor
//

public AktBusLindaInGate(	String linda_host,		//linda host
				int linda_port,			//linda port
				String myname,			//gateway's kraft name
				int myport,			//server socket port
				boolean verbose)
	throws Exception
{
super();					//call superclass constructor
client=new LindaClient(linda_host,linda_port);	//create linda client
this.myport=myport;
this.myname=myname;
this.verbose=verbose;
}



//
// thread execution for AktBusLindaInGate
//

public void run()
{
AktMessageWithPath message_with_path;

try	{
	receiver=new AktBusReceiver(myport);		//create AktBusReceiver object
	} catch (Exception e)
		{
		print_message("ERROR: fail to create server socket at port#"+myport);
		print_message("Stopping Incoming gate...");
		return;
		}
print_message("Incoming gate running...");
while (true)
	try	{
		message_with_path=receiver.receive();			//wait for a message from remote
//		print_message("Incoming:"+message_with_path.message);
		handle_incoming_message(message_with_path);		//handle the incoming message
		} catch (Exception e)
			{
			print_message("In-gate got something wrong");
			}
}

//
// handle an incoming message for AktBusLindaInGate
//
private void handle_incoming_message(AktMessageWithPath message_with_path)
{
String sender_arg,receiver_arg,message_arg;
String sender_uri_string,sender_host_string,sender_path_string;
int sender_port;
URL temp_sender_uri;
String tuple;

try	{
	//
	// take care of possible leadin '/' in request path
	//
	if (message_with_path.path.charAt(0)=='/')					//check for leading '/' in request path
		{
		message_with_path.path=message_with_path.path.substring(1);		//trim the leading '/'
		}
	message_arg=message_with_path.message.get_simple_message();			//get message node text
	//
	// message is converted into msg/3
	// i.e. msg(sender,receiver,content)
	//
	sender_uri_string=message_with_path.message.get_sender();			//get sender URI as a string
	temp_sender_uri=new URL(sender_uri_string);
	sender_host_string=temp_sender_uri.getHost();					//get sender URI's host
	sender_port=temp_sender_uri.getPort();						//get sender URI's port
	sender_path_string=temp_sender_uri.getFile();					//get sender URI's path
	sender_arg="http('"+sender_host_string+"',"+sender_port+",'"+sender_path_string+"')";	//compose http/3 term
	receiver_arg=message_with_path.path;						//the receiver's name in the Linda space
	tuple="msg("+sender_arg+","+receiver_arg+","+"\""+message_arg+"\""+")";		//compose the term as a string
	print_message(tuple);
	client.connect();				//connect to Linda server
	client.set_timeout(1000);			//wait max 1000ms
	client.out(tuple);				//out tuple string to linda server
	client.disconnect();				//disconnect
	} catch (IOException e)
		{
		print_message("ERROR: cannot 'out' to Linda ");
		print_message("Stopping Incoming gate...");
		return;						//stop thread
		}
	catch (Exception e)
		{
		print_message("Invalid Prolog term");
		return;
		}
}



//
// print verbose message depending on debug flag
//
private void print_message(String s)
	{
	if (verbose)
		{
		System.out.println(s);
		}
	}
}



//
//the outgoing-gate class
//

class AktBusLindaOutGate extends Thread
{
LindaClient client;		//linda-client
String myname;			//gateways' full kraft name
int my_port;			//socket port no. on which the in-gate receives messages
boolean verbose;		//debug message flag

//
// constructor
//

public AktBusLindaOutGate(	String linda_host,		//linda server host
				int linda_port,			//linda server port
				String myname,			//gateway's kraft name
				int my_port,			//socket port
				boolean verbose)		//debug flag
	throws Exception
{
super();					//call superclass constructor
client=new LindaClient(linda_host,linda_port);	//create client that takes messages from Linda
this.myname=myname;
this.verbose=verbose;
}



//
// thread execution
//
public void run()
{
String message;
int i;

print_message("Outgoing gate running...");
try	{
	client.connect();
	} catch (IOException e)
		{
		print_message("ERROR: cannot connect to Linda ");
		print_message("Stopping Outgoing  gate...");
		return;
		}

//
// if ok, go into a loop that "in" from Linda & send to AKT
//
do	{
	message=null;
	try	{
		message=client.in("msg(_,http(_,_,_),_)");	//look for messages with a receiver of http/3
		if (message!=null)
			{
			print_message("from Linda:"+message);
			handle_outgoing_message(message);	//handle "in-ed" message
			}
		} catch (IOException e)
			{
			print_message("ERROR: 'in' has problem");
			}
	} while (message!=null);
print_message("Stopping Outgoing  gate...");
return;
}


//
// handle message received by "in"
//

private void handle_outgoing_message(String message)
{
AktMessage am;

TermInString msg_term,receiver_term;
String sender_string,sender_host_string,sender_path_string;
String receiver_host_string,receiver_path_string;
String content_string;
int receiver_port,sender_port;
URL sender_uri,receiver_uri;

try	{
	msg_term=new TermInString(message);				//make a term from the message string
	sender_string=msg_term.argument(1);				//get sender, receiver & content
	receiver_term=new TermInString(msg_term.argument(2));		//create http term from receiver's URI string
	receiver_host_string=receiver_term.argument(1);			//get component of receiver's URI
	receiver_port=Integer.parseInt(receiver_term.argument(2));
	receiver_path_string=receiver_term.argument(3);
	if (receiver_host_string.charAt(0)=='\'')			//if there is a ' in the beginning
		receiver_host_string=receiver_host_string.substring(1,receiver_host_string.length()-1);	//trim away ' at both ends
	if (receiver_path_string.charAt(0)=='\'')			//if there is a ' in the beginning
		receiver_path_string=receiver_path_string.substring(1,receiver_path_string.length()-1);	//trim away ' at both ends
	System.err.println("*** finish with receiver");
	content_string=msg_term.argument(3);
	sender_host_string=InetAddress.getLocalHost().getHostName();
	sender_port=my_port;									//socket at my_port!
	sender_path_string="/"+sender_string;
	sender_uri=new URL("http",sender_host_string,sender_port,sender_path_string);		//make sender's URI object
	receiver_uri=new URL("http",receiver_host_string,receiver_port,receiver_path_string);	//make receiver's URI object
	System.err.println("** ready to make AktMessage from:"+content_string);
	am=new AktMessage(sender_uri,receiver_uri,content_string);				//create AktMessage
	am.send();
	} catch (Exception e)
		{
		System.err.println(e.toString());
		print_message("Invalid Prolog term received");
		return;
		}
}


//
// print message depending on verbose flag
//
private void print_message(String s)
{
if (verbose)
	System.out.println(s);
}
}	//end class AktBusLindaOutGate

