import com.sun.xml.tree.*;
import org.xml.sax.*;
import org.w3c.dom.*;
import java.util.*;
import java.net.*;
import java.io.*;

/**
 * The FLogicMessage class, subclasses AKTMessage to create an AKTMessage with
 * FLogic content.
 *
 * Constructors:
 *
 *	FLogicMessage(String sender,String receiver,String query) throws MalformedURLException
 *
 *		Create an AktMessage with the specified sender & receiver URLs
 *		and the query as XML content.
 *		Throws MalformedURLException if either URL is invalid.
 *
 * Object Methods:
 *
 *	void createNode(void)
 */
public class FLogicMessage extends AktMessage {

	/**
	 * Constructs an FLogicMessage with the query and String URIs given
	 *
	 * @param String sender The sender of the message.
	 * @param String receiver The receiver of the message.
	 * @param String query The query
	 *
	 * @throws MalformedURLException From AKTMessage
	 */
    public FLogicMessage(String sender, String receiver, String query) throws MalformedURLException {
		super();
		Node q = createNode(query);
		this.appendChild(createDOM(this, new URL(sender), new URL(receiver), q));
	}

	/**
	 * Creates the XML Representation of the query, following the
	 * ontoresult schema.
	 *
	 * @param String query The FLogic query to represent in XML.
	 *
	 * @returns Node The query as XML, to be placed in the AKTMessage as content
	 */
	public Node createNode(String query) {
		// Tokenize the query on '.'
		StringTokenizer st = new StringTokenizer(query, ".");

		// Add the stylesheet for the
		this.appendChild(createProcessingInstruction("xml-stylesheet", "href=\"http://quail.csd.abdn.ac.uk:18080/akt-ontobroker/result-rdf.xsl\" type=\"text/xsl\""));

		// Create the top level element - see the RDF Schema
		Node q = this.createElement("onto:OntobrokerQuery");
		// Add the URL for the schema
		((ElementNode) q).setAttribute("xmlns:onto", "http://quail.csd.abdn.ac.uk:18080/akt-ontobroker/schema/ontoresult.xml#");

		Node hq = this.createElement("onto:hasFLogicQuery");
		q.appendChild(hq);

		// NEED TO ADD
		// A quantifier property to the XML
		// perhaps change hasVars to multiple hasVar.

		// For each query
		while (st.hasMoreTokens()) {
			String singleQuery = st.nextToken();
			System.out.println(singleQuery);

			// The query
			Node fl = this.createElement("onto:FLogicQuery");
			hq.appendChild(fl);

			// Has vars and a relationship
			Node flv = this.createElement("onto:hasFLogicVars");
			Node flr = this.createElement("onto:hasFLogicRel");

			// In FLogic vars are on the left of the <- and relationships on the right.
			String vars = singleQuery.substring(0, singleQuery.indexOf("<-")).trim();
			String rel = singleQuery.substring(singleQuery.lastIndexOf("<-")+2, singleQuery.length()).trim();

			fl.appendChild(flv);
			fl.appendChild(flr);
			flv.appendChild( this.createTextNode(vars) );
			flr.appendChild( this.createTextNode(rel) );
		}
		return q;
	}

}