//
// testing the AKT-Bus to Linda gateway
//

import java.io.*;

public class TestLindaGateway
{
static boolean verbose=false;

public static void main(String[] argv)
{
String config_file=null;			//config file name
String linda_host=null;				//linda host
int linda_port=0;				//linda port
String myname=null;				//my KRAFT name
int myport=0;					//my socket port to use
//boolean verbose=false;			//default verbose flag off
AktBusLindaGateway gate;			//the gateway object that does the real job

//
// process command line arguments
//
try	{
	int i=0;
	while (i<argv.length)
		{
		if (argv[i].equals("-c"))
			config_file=argv[++i];			//get config file
		else if (argv[i].equals("-lh"))
			linda_host=argv[++i];			//get host
		else if (argv[i].equals("-lp"))
			linda_port=Integer.parseInt(argv[++i]);	//get port no.
		else if (argv[i].equals("-mn"))
			myname=argv[++i];			//get myname
		else if (argv[i].equals("-mp"))
			myport=Integer.parseInt(argv[++i]);	//get my port to use
		else if (argv[i].equals("-v"))
			verbose=true;
		else throw new Exception();			//unknown parameter
		i++;
		}
	} catch (Exception e)
		{
		System.err.println("usage: java GatewayTest -c config_file -lh linda_host -lp linda_port -mn my_name -mp my_port");
		System.exit(0);
		}


if (linda_host==null || linda_port==0 || myname==null || myport==0)
	{
	System.err.println("ERROR: not enough parameter specified");
	System.err.println("usage: java -classpath .. TestLindaGateway -mn <myname> -mp <my-port> -lh <linda-host> -lp <linda-port>");
	System.exit(4);
	}

print_message("Socket-based AKT-Bus-Linda gateway");
print_message("--------------------------");
print_message("linda host:"+linda_host);
print_message("linda port:"+linda_port);
print_message("my name:"+myname);
print_message("my port:"+myport);

//
// ready to start gateway
//
try	{
	gate=new AktBusLindaGateway(linda_host,linda_port,myname,myport,verbose);	//create gateway
	gate.work();									//start gateway
	} catch (Exception e)
		{
		print_message("ERROR in creating/running gateway");
		}
}



//
// print message depending on verbose flag
//
private static void print_message(String s)
	{
	if (verbose)
		System.out.println(s);
	}
}

