%%% File:     agent.pl
%%% Author:   Stephen Potter <stephenp@gairsay.aiai.ed.ac.uk>
%%% Created:  Fri Aug 10 2001

%%% simple agent program - sends a query to a broker, and
%%% waits for the response.


:-use_module('../src/absl').
%:-use_module(library('linda/client')).
:-use_module(library(lists)).
:-use_module(library(sockets)).
:-use_module(library(charsio)).

broker('http://cara.aiai.ed.ac.uk:').

% Port is the port of the socket to be opened on the agent host
% to accept the broker's response.
% Query is assumed to some atom or predicate.

seedp(5006).

testagent(SP,P) :-
	Target is SP + 5,
	tagent(SP,Target,P).

tagent(I,M,LP):-
	I<M,
	%seedp(SP),
	%P is SP+I,
	I1 is I+1,
	name(I,N),
	name(query,Q),
	append(Q,N,QS),
	name(Qu,QS),
	get_q(Qu,Query),
	agent(I,Query,LP),
	tagent(I1,M,LP).
tagent(I,I,LP):-
	seedp(SP),
	P is SP+I,
	agent(P,close,LP).

get_q(Q,ask(Q)).

make_content(Content,env('septag',[],Content)).



agent(Port,Query,LP):-
	% open connection for response: do this first to ensure
	% that (a) port is valid and (b) port will be open by the
	% time that the response is generated!

	%open_conn(Port,_,Socket),

	broker(BA),
	name(BA,BAN),
	name(LP,LPN),
	append(BAN,LPN,Broker),
	%Broker="http://cara.aiai.ed.ac.uk:8765", % need to edit this!

	current_host(X),
	name(X,H),
	append("http://",H,H1),
	append(H1,":",Agent),

	name(Port,P),
	append(Agent,P,AU),

	%name(Query,Q),
	make_content(Query,Cont),
	compose_message(AU,Broker,Cont,Msg),
	write(Msg),
	send_message(Msg).

	% wait for response:
	%get_message(Socket,Resp),
	
	% extract content:
	%get_env_attrib_value(Resp, 'aktbus:hasSimpleMessage',_,[Content|_]),
	%name(PSM, Content),
	%nl,write('PSM='),
	%write(PSM),nl,
	%close_conn(Socket).


ta(Port,Query,LP):-
	% open connection for response: do this first to ensure
	% that (a) port is valid and (b) port will be open by the
	% time that the response is generated!

	%open_conn(Port,_,Socket),

	broker(BA),
	name(BA,BAN),
	name(LP,LPN),
	append(BAN,LPN,Broker),
	%Broker="http://cara.aiai.ed.ac.uk:8765", % need to edit this!

	current_host(X),
	name(X,H),
	append("http://",H,H1),
	append(H1,":",Agent),

	name(Port,P),
	append(Agent,P,AU),

	%name(Query,Q),
	%ser(Query,Cont),
	%make_content(Query,Cont),
	%compose_message(AU,Broker,Cont,Msg),

	quote_atoms(Query,EQ),
	write_to_chars(EQ,CQ),

	escape_specials(CQ,ECQ,[]),
	
	compose_message(AU,Broker,ECQ,Msg),
	write(Msg),
	send_message(Msg),
	receive_message(Port,Reply),
	write(Reply).


quote_atoms(Q,Q):-
	(var(Q);number(Q)).
quote_atoms(Q,E):-
	atom(Q),
	esc([Q],[E]).
quote_atoms((A,B),(E1,E2)):-
	quote_atoms(A,E1),
	quote_atoms(B,E2).
quote_atoms(Query,EQ):-
	nonvar(Query),
	Query =..[F|Args],
	esc(Args,AL),
	EQ =..[F|AL].

esc([],[]).
esc([A|R],[N|AT]):-
	nonvar(A),
	\+atom(A),
	A =..[F|Args],
	esc(Args,AL),
	N =..[F|AL],
	esc(R,AT).
esc([A|R],[N|AT]):-
	nonvar(A),
	name(A,AS),
	append("\'",AS,T1),
	append(T1,"\'",T2),
	name(N,T2),
	esc(R,AT).
esc([A|R],[A|T]):-
	(var(A);number(A)),
	esc(R,T).


escape_specials([]) --> "".
escape_specials([46|T]) -->
	"\\.",
	escape_specials(T).
escape_specials([X|T]) -->
	[X],
	escape_specials(T).


serialise(Content, Terms):-
	var(Terms),
	!,
	xmls(Content,Terms,[]),
	%writeq(Terms).
	format(Terms,[]).

serialise(Content,Terms):-
	parse_sxml(Content,Terms,[]).

ser(Content,Terms1):-
	var(Terms1),
	!,
	numbervars(Term,0,_),
	xmyp1(Content,Terms,[]),
	format(Terms,[]),
	Terms1 = Terms.
	%name(Terms1,Terms).
ser(Content,Terms):-
	xmypr1(Content,Terms,[]).


xmls([]).
xmls([H]) -->
	!,
	xmls(H).
xmls([H|T]) -->
	!,
	
	xmls(H),",",
	xmls(T).



xmls(X) -->
	{var(X)},
	!,
	"[env(var,[],[])]".

xmls(X) -->
	{atomic(X),name(X,N)},
	!,
	"[env(atom,[],",N,")]".

xmls((X,Y)) -->
	!,
	xmls(X),",",
	xmls(Y).

xmls([X|T]) -->
	{X =.. ['.',A|Args],xmls(A,H,[]),xmls(Args,As,[])},
	!,
	"[env(.,[],[env(.,[],[",H,",",As,")]])])]".

xmls(X) -->
	{X =.. [H|Args],name(H,N),xmls(H,Fun,[]),xmls(Args,As,[])},
	!,
	"[env(term,[],[env(",N,",[],[",As,"])])]".

xmls((A:-B)) -->
	{xmls(A,L1,[]),xmls(B,L2,[])},
	!,
	%xmls(A),",",
	"[env(':-',[],[",L1,",",L2,"])]".


myp1(X) -->
	"env(prolog,[],[", myp(X), "])".

myp([]).
myp(X) -->
	{atom(X),name(X,N)},
	!,
	"<atom>",
	N,
	"</atom>".
myp(X) -->
	{var(X),atom_concat('V',X,VN),name(VN,N)},
	!,
	"<var>",
	N,
	"</var>".

myp([H])-->
	!,
	myp(H).

myp([H|T]) -->
	!,
	"[.,",
	myp(H),
	",",
	myp(T),
	"]".

myp(arg([])) --> "".
myp(arg([H|T])) -->
	!,
	"<arg>",myp(H),
	"</arg>",
	myp(arg(T)).

myp(X) -->
	{X =.. [F|Args],
	name(F,FN)},
	!,
	"<pred>",FN,
	myp(arg(Args)),
	"</pred>".


mypr1(X) --> "env(prolog,[],[",mypr(X),"])".

mypr(X) --> "<pred>", atomey(F), {\+end_tag(F)}, mypr(arg(A)), "</pred>", {append([F],A,FL),X=..FL}.
mypr(arg(A)) --> "<arg>", mypr(Arg), "</arg>", mypr(arg(Z)), {append([Arg],Z,A)}.
mypr(arg([])) --> "". 
mypr(X) --> "<atom>", atomey(X), "</atom>".
mypr(X) --> "<var>", atomey(X), "</var>".


atomey(Z) --> [C], {name(Z,[C])}.
atomey(Z) --> %[C], {\+[C]="<"}, 
	atomey(D), [C], {name(H,[C]), atom_concat(D,H,Z)}.
%atomey(Z) --> [C], {name(Z,[C]), \+Z='<'}.
atomey('') --> "".

end_tag(Z) :- 
	name(Z,N),
	append(B,C,N),
	append(A,"</",B).

mypro(Z) --> "[.,", mypr(A),",", mypr(B), "]", {append(A,B,Z)}.
mypro(Z) --> "[",atomy(X),",",mypr(A),"]", {Z=..[X,A]}.



%parse_sxml((A,B)) --> parse_sxml(A), ",", parse_sxml(B).
parse_sxml((A:-B)) --> "[env(':-',[],[", parse_sxml(A), ",", parse_sxml(B), "])]".
parse_sxml(Z) --> "[env(var,[],[]", {var(Z)}.
parse_sxml(Z) --> 
	"[env(atom,[],", atomy(A), ")],",
	parse_sxml(B),
	{(is_list(B),append([A],B,Z));append([A],[B],Z)}.

parse_sxml(Z) --> "[env(atom,[],", atomy(Z), ")]".

parse_sxml(Z) --> "[env(term,[],", parse_sxml(Z), ")]".
parse_sxml(Z) --> "[env(", atomy(F), ",[],[",parse_sxml(As),"])]", {append([F],As,FL), Z=..FL}.
parse_sxml([]).
%parse_sxml((A,B)) --> parse_sxml(A), ",", parse_sxml(B).

parse_r_sxml(Z) --> "[env(atom,[],", atomy(A), ")],", parse_r_sxml(B), {append([A],B,Z)}.
parse_r_sxml([Z]) --> "[env(atom,[],", atomy(Z), ")]".

%atomthing(_).
%atomthing(X) --> [[C]],{name(X,[C])}.
atomthing(X) --> atomy(C),{name(X,C),atom(X)}.

atomy(Z) --> [C], atomy(D), {name(H,[C]), atom_concat(H,D,Z)}. 
atomy(Z) --> [C], {name(Z,[C])}.



%----------------


xmyp1(X) -->
	{numbervars(X,0,_)},
	"env(prolog,[],[", xmyp(X), "])".

%xmyp(X) -->
%	{var(X),numbervars(X,0,_),X='$VAR'(Y), name(Y,N), append("V",N,VN)},
%	!,
%	"env(var,[],[",
%	VN,
%	"])".

xmyp(X) -->
	{var(X),%numbervars(X,0,_),
	X =.. [F|Args],
	name(F,FN)},
	!,
	"env(pred,[Name=",FN,"],[",
	xmyp(arg(Args)),
	"])".

xmyp((A,B)) -->
	!,
	xmyp(A),
	",",
	xmyp(B).

%xmyp([]).
xmyp([])-->
	!,
	"env(list,[],[[]])".

xmyp([H|T]) -->
	!,
	"env(list,[],[",
	xmyp(H),
	",",
	xmyp(T),
	"])".

xmyp(X) -->
	{atom(X),name(X,N)},
	!,
	"env(atom,[],[",
	N,
	"])".
xmyp(X) -->
	{number(X),name(X,N)},
	!,
	"env(number,[],[",
	N,
	"])".



xmyp(0,arg(_)) --> "".
xmyp(N,arg([H|T])) -->
	!,
	{name(N,Ar)},
	"env(arg,[Arg=",Ar,"],[",xmyp(H),
	"])",
	{(N>1,Comma=",");Comma=""},
	Comma,
	{N1 is N-1},
	xmyp(N1,arg(T)).

xmyp(X) -->
	{X =.. [F|Args],
	functor(X,F,Ar),
	name(Ar,AN),
	name(F,FN)},
	!,
	"env(pred,[Name=",FN,",Arity=",AN,"],[",
	xmyp(Ar,arg(Args)),
	"])".


xmypr1(X) --> "env(prolog,[],[",xmypr(X),"])".



%xmypr(argL([])) --> "".
%xmypr(argL([Arg])) --> xmypr(arg(Arg)).
%xmypr(argL(A)) -->  xmypr(argL(Z)), ",", xmypr(arg(Arg)), {append(Z,[Arg],A)}.


%xmypr(arg([])) --> "".

%xmypr(arg([Arg])) --> "env(arg,[],[", xmypr(Arg), "])".

axmypr(0,arg([])) --> "".
axmypr(Ar,arg(A)) --> {Ar>0}, "env(arg,[Arg=", xnumber(AN),{\+xend_tag(AN),AN=Ar},"],[", xmypr(Arg), "])", {A1 is Ar-1}, join(A1), axmypr(A1,arg(Z)), {append([Arg],Z,A)}.


join(0) --> "".
join(Ar) --> {Ar>0},",".



%xmypr(arg([])) --> "". 

xmypr(X) --> "env(atom,[],[", xatomey(X), {\+xend_tag(X)}, "])".
xmypr(X) --> "env(number,[],[", xnumber(X), {\+xend_tag(X)}, "])".
%xmypr(X) --> "env(var,[],[", xatomey(X), {\+xend_tag(X)}, "])".
xmypr([]) --> "env(list,[],[[]])".
xmypr(X) --> "env(list,[],[", xmypr(H), ",", xmypr(T), "])", {append([H],T,X)}.

xmypr(Z) --> 
	"env(pred,[Name=", xatomey(F), {\+xend_tag(F)},",Arity=", xnumber(Ar),{\+xend_tag(Ar)},"],[", axmypr(Ar,arg(A)), "])", {append([F],A,FL),Z=..FL}.

%xmypr(Z) --> 
%	"env(pred,[Name=", xatomey(F), {\+xend_tag(F)},"],[", axmypr(arg(A)), "])", 
%	join,
%	xmypr(N),
%	{append([F],A,FL),X=..FL, ((nonvar(N),Z=(X,N));(var(N),Z=X))}.

xmypr((A,B)) --> xmypr(A), ",", xmypr(B).

xnumber(Z) --> [C], {Z is C - 48}.
xnumber(Z) --> xnumber(X),[C], {Z is (X*10)+(C-48)}.


xatomey(Z) --> [C], {name(X,[C]),((number(X),lookupn(X,Z));(\+number(X),Z=X))}.
xatomey(Z) --> %[C], {\+[C]="<"}, 
	xatomey(D), [C], {name(X,[C]), ((number(X),lookupn(X,H));(\+number(X),H=X)), atom_concat(D,H,Z)}.
%xatomey(Z) --> [C], {name(Z,[C]), \+Z='<'}.
%xatomey('') --> "".

xend_tag(Z) :- 
	name(Z,N),
	append(B,C,N),
	append(A,"])",B).

lookupn(0,'0').
lookupn(1,'1').
lookupn(2,'2').
lookupn(3,'3').
lookupn(4,'4').
lookupn(5,'5').
lookupn(6,'6').
lookupn(7,'7').
lookupn(8,'8').
lookupn(9,'9').
