%%% File:     broker.pl
%%% Author:   Stephen Potter <stephenp@gairsay.aiai.ed.ac.uk>
%%% Created:  Fri Aug 10 2001

%
% This is the code to implement a brokering system.
% The broker should be able to receive a request (in the form
% of some logical proposition) and then, using the facts
% available to it from a linda server, try to construct a proof
% of this proposition.
% If a proof is possible, the broker returns this to the querying
% agent, else failure is reported somehow.
%
% The original query is assumed to arrive via the AKTknowledge Bus...?
%

:-use_module('../src/abslm').
:-use_module(library('linda/client')).
:-use_module(library(lists)).
:-use_module(library(sockets)).


%%%
%%% Port is that used for incoming query connection.
%%%
broker(Port):-
	L=35219,
	linda_client('kerrera.aiai.ed.ac.uk':L), %note syntax!
	open_conn(Port,_,Socket), % msg is string?
	write('Port '), write(Port), write(' opened'), nl,
	wait_for_query(Socket),
	close_conn(Socket),
	close_client.


% might need to extend this(!):
well-formed-query(_).


% 
% query arrives as a XML-wrapped package.
%
wait_for_query(Socket):-
	write('Linda Broker Listening...'),nl,
	get_message(Socket,Msg),

%	get_env_attrib_value(Msg, 'kraft:Message',_,[Content|_]),
	get_env_attrib_value(Msg, 'aktbus:hasSimpleMessage',_,[Content|_]),

	% for this to work, the sender must have an open port too?
	% (and this information must be included in the senderUri)

	get_env_attrib_value(Msg, 'aktbus:hasSenderURI',_,[SenderURI|_]),
	get_env_attrib_value(Msg, 'aktbus:hasReceiverURI',_,[BrokerURI|_]),

	%get_env_attrib_value(Msg, 'aktbus:hasDateString',_,[Date|_]),
	%get_env_attrib_value(Msg, 'aktbus:hasTimeString',_,[Time|_]),
	%nl,write(Content),nl,%write(SenderURI),
	name(Query, Content),
	name(Agent, SenderURI),
	well-formed-query(Query),
	write('Received query from '), write(Agent), write(' = '),
	nl,write(Query),nl,
	(
	    (
		Query = kill_broker, % privileged atom kills broker.
		nl,write('Broker terminated.'),nl,
		true
	    );

	    (

		write('Accessing Linda...'),nl,

	    % check the linda server if the Query can be satisfied
	    % [query 'proof' mechanism goes here].
	    % note: _noblock: if the fact is not there, the system
	    % does not wait for it!

		(
		    (rd_noblock(fakt(PSM,_,_,Query)),
		    nl,write('retrieved: '),write(PSM),nl);
	    
	    % else return dummy value:
		    (nl,write('no suitable problem-solvers.'),nl,
		    PSM=no)
		),

		nl,write('respond'),
		name(PSM,P),
		compose_message(BrokerURI,SenderURI,P,Resp),
		send_message(Resp),
		wait_for_query(Socket) %loop the broker
	
	    )

	).	
		



