%%% File:     listen.pl
%%% Author:   Stephen Potter <stephenp@gairsay.aiai.ed.ac.uk>
%%% Created:  Thu Aug  9 2001

%%% intended as a first stab as a port 'listener' interface
%%% to linda.
%%% 

:-use_module('../src/absl').
:-use_module(library('linda/client')).
:-use_module(library(lists)).


%%%
%%% linda_gate initiates a client process with the specified linda
%%% server, and then opens its own connection on the host maching
%%% (with port Port) and waits for communications.
%%% When it arrives, a message is an xml-rdf wrapped thing; from
%%% this, the content, sender and time/date are extracted, and
%%% used to compose a fact, which is then added to the linda server.
%%% The process then waits for the next message, and repeats these
%%% steps until it receives a message (from any sender) containing
%%% as content the atom 'close'.

linda_gate(Port):-
	%L=35219, % linda server port.
	%linda_client('kerrera.aiai.ed.ac.uk':L), %note syntax!
	%open_conn(Port,_,Socket), % msg is string?
	create_aktbus_receiver(Port,Socket),
	write('Port '), write(Port), write(' opened'), nl,
	wait_for_msg(Socket),
	!,
	close_conn(Socket).
	%close_client.

wait_for_msg(Socket):-
%	write('Linda Gateway Open (Port:'), write(Port), write(')'), nl,
	receive_message(Socket,Msg),
	write(Msg),nl,
%	get_env(Msg, 'kraft:Message',_,[Content|_]),
%	get_env(Msg, 'aktbus:hasContent',_,[Content|_]),
	get_env(Msg, 'aktbus:hasSimpleMessage',_,[Content|_]),
	get_env(Msg, 'aktbus:hasSenderURI',_,[SenderURI|_]),
	get_env(Msg, 'aktbus:hasDateString',_,[Date|_]),
	get_env(Msg, 'aktbus:hasTimeString',_,[Time|_]),
	%extract_content(Msg,Content),
	%write('****************************'),nl,
	nl,write(Content),nl,write(SenderURI),
	name(Content,Cont),

	open('temp.pl', write, Stream),
	format(Stream, Cont, []),

	close(Stream),

	open('temp.pl', read, Stream2),
	read(Stream2,Term),
	write(Term),
	close(Stream2),

	Term,

	%ser(I,Cont),
	%nl,write(I),nl,
	%I,
	%name(C, Content),
	%C = ok,
	%name(S, SenderURI),
	%name(D, Date),
	%name(T, Time),
	!,
	%((\+ C = close,
	%(C,(write('ok'));write('no')),
	%await(1000000),
	%wait_for_msg(Socket));
	%true).
	wait_for_msg(Socket).

ask(Q):-
	write('asking:'), write(Q), nl.
	%name(X,N,Q),
	%number(N),
	%(N mod 2)=0. 
ask(_):- fail.

await(0).
await(N):-
	N>0,
	N1 is N-1,
	await(N1).


ser(Content,Terms1):-
	var(Terms1),
	!,
	xmyp1(Content,Terms,[]),
	format(Terms,[]),
	name(Terms1,Terms).
ser(Content,Terms):-
	xmypr1(Content,Terms,[]).

myp1(X) -->
	"env(prolog,[],[", myp(X), "])".

myp([]).
myp(X) -->
	{atomic(X),name(X,N)},
	!,
	"<atom>",
	N,
	"</atom>".
myp('$VAR'(X)) -->
	{atom_concat('V',X,VN),name(VN,N)},
	!,
	"<var>",
	N,
	"</var>".

myp([H])-->
	!,
	myp(H).

myp([H|T]) -->
	!,
	"[.,",
	myp(H),
	",",
	myp(T),
	"]".

myp(arg([])) --> "".
myp(arg([H|T])) -->
	!,
	"<arg>",myp(H),
	"</arg>",
	myp(arg(T)).

myp(X) -->
	{X =.. [F|Args],
	name(F,FN)},
	!,
	"<pred>",FN,
	myp(arg(Args)),
	"</pred>".


mypr1(X) --> "env(prolog,[],[",mypr(X),"])".

mypr(X) --> "<pred>", atomey(F), {\+end_tag(F)}, mypr(arg(A)), "</pred>", {append([F],A,FL),X=..FL}.
mypr(arg(A)) --> "<arg>", mypr(Arg), "</arg>", mypr(arg(Z)), {append([Arg],Z,A)}.
mypr(arg([])) --> "". 
mypr(X) --> "<atom>", atomey(X), "</atom>".
mypr(X) --> "<var>", atomey(X), "</var>".


atomey(Z) --> [C], {name(Z,[C])}.
atomey(Z) --> %[C], {\+[C]="<"}, 
	atomey(D), [C], {name(H,[C]), atom_concat(D,H,Z)}.
%atomey(Z) --> [C], {name(Z,[C]), \+Z='<'}.
atomey('') --> "".

end_tag(Z) :- 
	name(Z,N),
	append(B,C,N),
	append(A,"</",B).





%---------------

xmyp1(X) -->
	"env(prolog,[],[", xmyp(X), "])".

xmyp([]).
xmyp(X) -->
	{atomic(X),name(X,N)},
	!,
	"env(atom,[],[",
	N,
	"])".
xmyp(X) -->
	{var(X),atom_concat('V',X,VN),name(VN,N)},
	!,
	"env(var,[],",
	N,
	")".

xmyp([H])-->
	!,
	xmyp(H).

xmyp([H|T]) -->
	!,
	"[.,",
	xmyp(H),
	",",
	xmyp(T),
	"]".

xmyp(arg([])) --> "".
xmyp(arg([H|T])) -->
	!,
	"env(arg,[],[",xmyp(H),
	"])",
	{(\+T=[],Comma=",");Comma=""},
	Comma,
	xmyp(arg(T)).

xmyp(X) -->
	{X =.. [F|Args],
	name(F,FN)},
	!,
	"env(pred,[Name=",FN,"],[",
	xmyp(arg(Args)),
	"])".


xmypr1(X) --> "env(prolog,[],[",xmypr(X),"])".



%xmypr(argL([])) --> "".
%xmypr(argL([Arg])) --> xmypr(arg(Arg)).
%xmypr(argL(A)) -->  xmypr(argL(Z)), ",", xmypr(arg(Arg)), {append(Z,[Arg],A)}.


%xmypr(arg([])) --> "".

%xmypr(arg([Arg])) --> "env(arg,[],[", xmypr(Arg), "])".
axmypr(arg(A)) --> "env(arg,[],[", xmypr(Arg), "])", join, axmypr(arg(Z)), {append([Arg],Z,A)}.
axmypr(arg([])) --> "".

join --> ",".
join --> "".


%xmypr(arg([])) --> "". 

xmypr(X) --> "env(atom,[],[", xatomey(X), {\+xend_tag(X)}, "])".
xmypr(X) --> "env(var,[],[", xatomey(X), {\+xend_tag(X)}, "])".

xmypr(X) --> "env(pred,[Name=", xatomey(F), {\+xend_tag(F)},"],[", axmypr(arg(A)), "])", {append([F],A,FL),X=..FL}.




xatomey(Z) --> [C], {name(Z,[C])}.
xatomey(Z) --> %[C], {\+[C]="<"}, 
	xatomey(D), [C], {name(H,[C]), atom_concat(D,H,Z)}.
%xatomey(Z) --> [C], {name(Z,[C]), \+Z='<'}.
%xatomey('') --> "".

xend_tag(Z) :- 
	name(Z,N),
	append(B,C,N),
	append(A,"])",B).
