import aktbus.core.*;
import aktbus.core.fipa.*;

import java.io.*;
import java.util.*;
import java.net.*;

import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import com.hp.hpl.mesa.rdf.jena.common.PropertyImpl;

public class TestFipaMessage {

public static void main(String argv[]) {

    if (argv.length!=1)
	{
	    System.err.println("usage: java TestFipaMessage <receiver>");
	    System.exit(0);
	}
    try {
	StringWriter sw2 = new StringWriter();

	//Set up file reader to read aktmessage
	FileReader in = new FileReader("fipaexample2.rdf");
	int temp = in.read();
	while (temp != -1) {
	    sw2.write(temp);
	    temp = in.read();
	}
	sw2.close(); 

	RDFFipaMessage msg = new RDFFipaMessage(sw2.toString()); 

	/*	RDFFipaMessage msg = new RDFFipaMessage(new URL("http://sender.com:1234"), new URL("http://receiver.com:1234"),
		"(query-if :sender (agent-identifier :name fred :addresses (sequence http://hawk:7778/acc http://hawk:1234/acc ))          :receiver (set (agent-identifier :name john :addresses (sequence http://hawk:7778/acc http://hawk:1234/acc) ) (agent-identifier :name bob :addresses (sequence http://hawk:7778/acc http://hawk:1234/acc) )) :content \"some content here\" :ontology some ontology :reply-with some reply-with details :reply-by some reply-by details :conversation-id http://some.host.org:1234/abc :protocol query-protocol )"); 
	*/

	//RDFFipaMessage msg = new RDFFipaMessage();
		
	sw2 = new StringWriter();
	msg.ugly_write(sw2);
	sw2.close();	
	System.out.println("\n\n" + sw2.toString());

	System.out.println("\nReceiver: " + msg.get_receiver()); 
	System.out.println("Sender: " + msg.get_sender());
	System.out.println("Content (whole fipa message): " + msg.get_simple_message());

	System.out.println("\nReplacing receiver, sender and content with strings...");
	msg.set_receiver(argv[0]);
	msg.set_sender("http://new.sender.com:1234");
	
	System.out.println("Receiver: " + msg.get_receiver()); 
	System.out.println("Sender: " + msg.get_sender());
	System.out.println("Content: " + msg.get_simple_message());

	System.out.println("\nReplacing sender and receiver with URIs...");
	msg.set_sender(new URL("http://raven.csd.abdn.ac.uk:1234"));
	msg.set_receiver(new URL("http://raven.csd.abdn.ac.uk:4000"));
	System.out.println("Receiver: " + msg.get_receiver()); 
	System.out.println("Sender: " + msg.get_sender());

	System.out.println("\nPeformative: " + msg.get_performative());
	System.out.println("Fipa sender: " + msg.get_fipa_sender());
	System.out.println("Fipa receiver: " + msg.get_fipa_receiver());
	System.out.println("Fipa content: " + msg.get_fipa_content());
	System.out.println("Reply-to: " + msg.get_reply_to());
	System.out.println("In-reply-to: " + msg.get_in_reply_to());
	System.out.println("Language: " + msg.get_language());
	System.out.println("Encoding: " + msg.get_encoding());
	System.out.println("Ontology: " + msg.get_ontology());
	System.out.println("Reply-with: " + msg.get_reply_with());
	System.out.println("Reply-by: " + msg.get_reply_by());
	System.out.println("Conversation-id: " + msg.get_conversation_id());
	System.out.println("Protocol: " + msg.get_protocol());

	System.out.println("\nChanges fields...");
	msg.set_performative("inform");
	msg.set_fipa_sender("(agent-identifier :name Bill :addresses (sequence http://hawk.csd.abdn.ac.uk:1234 http://hawk.csd.abdn.ac.uk:4321))");
	msg.set_fipa_receiver("(set (agent-identifier :name Bill :addresses (sequence http://hawk.csd.abdn.ac.uk:1234 http://hawk.csd.abdn.ac.uk:4321)) (agent-identifier :name Harry :addresses (sequence http://raven.csd.abdn.ac.uk:2000)))");
	msg.set_fipa_content("new content");
	msg.set_in_reply_to("new in-reply-to details");
	msg.set_reply_to("new reply to...");
	msg.set_language("new language");
	msg.set_encoding("new encoding");
	msg.set_ontology("new ontology");
	msg.set_reply_with("new reply with");
	msg.set_reply_by("new reply by");
	msg.set_conversation_id("http://hawk.csd.abdn.ac.uk/new_id");
	msg.set_protocol("http://csd.abdn.ac.uk/~mmc/fipa-aktbus.xml#request-protocol");
	System.out.println("\nWhole FIPA message: " + msg.get_simple_message());

	System.out.println("\nSetting entire message...");
	msg.set_simple_message("(query-if :sender (agent-identifier :name fred :addresses (sequence http://hawk:7778/acc http://hawk:1234/acc ))          :receiver (set (agent-identifier :name john :addresses (sequence http://hawk:7778/acc http://hawk:1234/acc) ) (agent-identifier :name bob :addresses (sequence http://hawk:7778/acc http://hawk:1234/acc) )) :content \"some content here\" :reply-to some reply-to details :in-reply-to some in reply to details :language some language :encoding some encoding :ontology some ontology :reply-with some reply-with details :reply-by some reply-by details :conversation-id http://some.host.org:1234/abc :protocol query-protocol )");

	System.out.println("\nWhole FIPA message: " + msg.get_simple_message());

	System.out.println("\n\nPrinting with ugly_write method:");
	sw2 = new StringWriter();
	msg.ugly_write(sw2);
	sw2.close();	
	System.out.println("\n" + sw2.toString());

	System.out.println("\n\nPrinting with pretty_write method:");
	sw2 = new StringWriter();
	msg.pretty_write(sw2);
	sw2.close();	
	System.out.println("\n" + sw2.toString());

	msg.set_receiver(new URL("http://sol.csd.abdn.ac.uk:16512"));
	System.out.println("Receiver: " + msg.get_receiver()); 
	System.out.println("Sending message to " + msg.get_receiver());
	msg.send();


	}
	catch (IOException e)
		{
		System.err.println("Error in I/O...");
		System.err.println(e.toString());
		}
	 catch (Exception e)
		{
		System.err.println("***ERROR: "+e.toString());
		}
}

}
