package aktbus.core;

import java.io.*;
import java.util.*;

// Jena packages
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import com.hp.hpl.mesa.rdf.jena.common.PropertyImpl;

public class AktPrettyWriter {

    public AktPrettyWriter() {
    }

    public void writeMessage(Model model, Writer out) {
	try {

	out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
	out.write("<rdf:RDF xmlns:rdfs=\"" +RDFS.getURI() + "\" ");
	out.write("xmlns:rdf=\"" + RDF.getURI() + "\" ");
	out.write("xmlns:aktbus=\"" +  AKTVOCAB.getURI()  + "\">");

	ResIterator iter = model.listSubjectsWithProperty(RDF.type, AKTVOCAB.Container);
	Resource container = iter.next();
	iter.close();
	Resource header = container.getProperty(AKTVOCAB.hasHeader).getResource();
	Resource content = container.getProperty(AKTVOCAB.hasContent).getResource();
	out.write("<aktbus:Container");
	if (container.hasProperty(AKTVOCAB.id)) {
	out.write(" aktbus:id=");
	out.write("\"" +container.getProperty(AKTVOCAB.id).getString() +"\""); }
	out.write(">");

	writeHeader(header, out);
	writeContent(model, out);

	out.write("</aktbus:Container>");
	out.write("</rdf:RDF>");
	
	}
	catch (IOException e)
		{
		System.err.println("Error in I/O...");
		System.err.println(e.toString());
		}
	 catch (Exception e)
		{
		System.err.println("***ERROR: "+e.toString());
		}
    }

    public void writeHeader(Resource header, Writer out) throws IOException {

	out.write("<aktbus:hasHeader>");
	out.write("<aktbus:SimpleHeader>");

	try {
	    header.getProperty(AKTVOCAB.hasDateString);
	    out.write("<aktbus:hasDateString>");
	    out.write(header.getProperty(AKTVOCAB.hasDateString).getString());
	    out.write("</aktbus:hasDateString>");
	}
	catch (Exception e) {}
	    
	try {
	    header.getProperty(AKTVOCAB.hasTimeString);
	    out.write("<aktbus:hasTimeString>");
	    out.write(header.getProperty(AKTVOCAB.hasTimeString).getString());
	    out.write("</aktbus:hasTimeString>");
	}
	catch (Exception e) {}
	
	try {
	    header.getProperty(AKTVOCAB.hasSenderURI);
	    out.write("<aktbus:hasSenderURI>");
	    out.write(header.getProperty(AKTVOCAB.hasSenderURI).getString());
	    out.write("</aktbus:hasSenderURI>");
	}
	catch (Exception e) {}
	
	try {
	    header.getProperty(AKTVOCAB.hasReceiverURI);
	    out.write("<aktbus:hasReceiverURI>");
	    out.write(header.getProperty(AKTVOCAB.hasReceiverURI).getString());
	    out.write("</aktbus:hasReceiverURI>");
	}
	catch (Exception e) {}
	
	out.write("</aktbus:SimpleHeader>");
	out.write("</aktbus:hasHeader>");	
    }

    public void writeContent(Model model, Writer out) throws IOException, RDFException {

	ResIterator iter = model.listSubjectsWithProperty(RDF.type, AKTVOCAB.Container);
	Resource container = iter.next();
	iter.close();
	Resource content = container.getProperty(AKTVOCAB.hasContent).getResource();

	out.write("<aktbus:hasContent>");
	out.write("<aktbus:SimpleContent>");

	try {
	    content.getProperty(AKTVOCAB.hasSimpleMessage);
	    out.write("<aktbus:hasSimpleMessage rdf:parseType=\"Literal\">");
	    out.write(content.getProperty(AKTVOCAB.hasSimpleMessage).getString());
	    out.write("</aktbus:hasSimpleMessage>");
	}
	catch (Exception e) {}

	out.write("</aktbus:SimpleContent>");
	out.write("</aktbus:hasContent>");
    }

    public void prettyWriteMessage(Model model, Writer out) {
	
	try {

	out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?> \n");
	out.write("<rdf:RDF xmlns:rdfs=\"" +RDFS.getURI() + "\" \n");
	out.write("\txmlns:rdf=\"" + RDF.getURI() + "\" \n");
	out.write("\txmlns:aktbus=\"" +  AKTVOCAB.getURI()  + "\">\n");

	ResIterator iter = model.listSubjectsWithProperty(RDF.type, AKTVOCAB.Container);
	Resource container = iter.next();
	iter.close();
	Resource header = container.getProperty(AKTVOCAB.hasHeader).getResource();
	Resource content = container.getProperty(AKTVOCAB.hasContent).getResource();

	out.write("<aktbus:Container");
	if (container.hasProperty(AKTVOCAB.id)) {
	out.write(" aktbus:id=");
	out.write("\"" +container.getProperty(AKTVOCAB.id).getString() +"\""); }
	out.write(">\n");

	prettyWriteHeader(header, out);
	prettyWriteContent(model, out);

	out.write("</aktbus:Container>\n");
	out.write("</rdf:RDF>\n");
	
	}
	catch (IOException e)
		{
		System.err.println("Error in I/O...");
		System.err.println(e.toString());
		}
	 catch (Exception e)
		{
		System.err.println("***ERROR: "+e.toString());
		}
    }

    public void prettyWriteHeader(Resource header, Writer out) throws IOException {
	out.write("  <aktbus:hasHeader>\n    <aktbus:SimpleHeader>\n");

	try {
	    header.getProperty(AKTVOCAB.hasDateString);
	    out.write("      <aktbus:hasDateString>");
	    out.write(header.getProperty(AKTVOCAB.hasDateString).getString());
	    out.write("</aktbus:hasDateString>\n");
	}
	catch (Exception e) {}
	    
	try {
	    header.getProperty(AKTVOCAB.hasTimeString);
	    out.write("      <aktbus:hasTimeString>");
	    out.write(header.getProperty(AKTVOCAB.hasTimeString).getString());
	    out.write("</aktbus:hasTimeString>\n");
	}
	catch (Exception e) {}
	
	try {
	    header.getProperty(AKTVOCAB.hasSenderURI);
	    out.write("      <aktbus:hasSenderURI>");
	    out.write(header.getProperty(AKTVOCAB.hasSenderURI).getString());
	    out.write("</aktbus:hasSenderURI>\n");
	}
	catch (Exception e) {}
	
	try {
	    header.getProperty(AKTVOCAB.hasReceiverURI);
	    out.write("      <aktbus:hasReceiverURI>");
	    out.write(header.getProperty(AKTVOCAB.hasReceiverURI).getString());
	    out.write("</aktbus:hasReceiverURI>\n");
	}
	catch (Exception e) {}
	
	out.write("    </aktbus:SimpleHeader>\n");
	out.write("  </aktbus:hasHeader>\n");

    }

    public void prettyWriteContent(Model model, Writer out) throws IOException, RDFException {

	ResIterator iter = model.listSubjectsWithProperty(RDF.type, AKTVOCAB.Container);
	Resource container = iter.next();
	iter.close();
	Resource content = container.getProperty(AKTVOCAB.hasContent).getResource();
	
	out.write("  <aktbus:hasContent>\n");
	out.write("    <aktbus:SimpleContent>\n");

	try {
	    content.getProperty(AKTVOCAB.hasSimpleMessage);
	    out.write("      <aktbus:hasSimpleMessage rdf:parseType=\"Literal\">");
	    out.write(content.getProperty(AKTVOCAB.hasSimpleMessage).getString());
	    out.write("</aktbus:hasSimpleMessage>\n");
	}
	catch (Exception e) {}

	out.write("    </aktbus:SimpleContent>\n");
	out.write("  </aktbus:hasContent>\n");
    }

}
