package aktbus.core.fipa;

import aktbus.core.*;

import java.io.*;
import java.util.*;

// Jena packages
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import com.hp.hpl.mesa.rdf.jena.common.PropertyImpl;

public class FipaPrettyWriter extends AktPrettyWriter {

    public FipaPrettyWriter() {
    }

    public void writeMessage(Model model, Writer out) {
	try {

	out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?> ");
	out.write("<rdf:RDF xmlns:rdfs=\"" +RDFS.getURI() + "\" ");
	out.write("xmlns:rdf=\"" + RDF.getURI() + "\" ");
	out.write("xmlns:aktbus=\"" +  AKTVOCAB.getURI()  + "\" ");
	out.write("xmlns:fipa=\"" + FIPAVOCAB.getURI() + "\">");

	ResIterator iter = model.listSubjectsWithProperty(RDF.type, AKTVOCAB.Container);
	Resource container = iter.next();
	iter.close();
	Resource header = container.getProperty(AKTVOCAB.hasHeader).getResource();
	Resource content = container.getProperty(AKTVOCAB.hasContent).getResource();

	out.write("<aktbus:Container");
	if (container.hasProperty(AKTVOCAB.id)) {
	out.write(" aktbus:id=");
	out.write("\"" +container.getProperty(AKTVOCAB.id).getString() +"\""); }
	out.write(">");

	writeHeader(header, out);
	writeContent(model, out);

	out.write("</aktbus:Container>");
	out.write("</rdf:RDF>");
	
	}
	catch (IOException e)
		{
		System.err.println("Error in I/O...");
		System.err.println(e.toString());
		}
	 catch (Exception e)
		{
		System.err.println("***ERROR: "+e.toString());
		}
    }

  
    public void writeContent(Model model, Writer out) throws IOException {

	out.write("<aktbus:hasContent>");

	try {
	    ((RDFFipaMessage)model).get_performative();
	    out.write("<fipa:" + ((RDFFipaMessage)model).get_performative());	   	   

	    try {
		Resource content = ((RDFFipaMessage)model).get_content();
		content.getProperty(FIPAVOCAB.id);
		out.write(" fipa:id=");
		out.write("\"" + content.getProperty(FIPAVOCAB.id).getString() +"\""); 
	    }
	    catch (Exception f) {}

	    out.write(">");
	  
	    
	    try {
			if (! (((RDFFipaMessage)model).get_fipa_sender().equals("")) ) {
			    out.write("<fipa:sender>");
			    out.write(((RDFFipaMessage)model).get_fipa_sender());
			    out.write("</fipa:sender>");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_fipa_receiver().equals("")) ) {
			    out.write("<fipa:receiver>");
			    out.write(((RDFFipaMessage)model).get_fipa_receiver());
			    out.write("</fipa:receiver>");
			}
		    }
		    catch (Exception f) {}
		    
		    try {
			if (! (((RDFFipaMessage)model).get_fipa_content().equals("")) ) {
			    out.write("<fipa:content rdf:parseType=\"Literal\">");
			    out.write(((RDFFipaMessage)model).get_fipa_content());
			    out.write("</fipa:content>");
			}
		    }
		    catch (Exception f) {}
	     
		    try {
			if (! (((RDFFipaMessage)model).get_reply_to().equals("")) ) {
			    out.write("<fipa:reply-to>");
			    out.write(((RDFFipaMessage)model).get_reply_to());
			    out.write("</fipa:reply-to>");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_in_reply_to().equals("")) ) {
			    out.write("<fipa:in-reply-to>");
			    out.write(((RDFFipaMessage)model).get_in_reply_to());
			    out.write("</fipa:in-reply-to>");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_language().equals("")) ) {
			    out.write("<fipa:language>");
			    out.write(((RDFFipaMessage)model).get_language());
			    out.write("</fipa:language>");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_encoding().equals("")) ) {
			    out.write("<fipa:encoding>");
			    out.write(((RDFFipaMessage)model).get_encoding());
			    out.write("</fipa:encoding>");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_ontology().equals("")) ) {
			    out.write("<fipa:ontology>");
			    out.write(((RDFFipaMessage)model).get_ontology());
			    out.write("</fipa:ontology>");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_reply_with().equals("")) ) {
			    out.write("<fipa:reply-with>");
			    out.write(((RDFFipaMessage)model).get_reply_with());
			    out.write("</fipa:reply-with>");
			}
		    }
		    catch (Exception f) {}
	    
		    try {
			if (! (((RDFFipaMessage)model).get_reply_by().equals("")) ) {
			    out.write("<fipa:reply-by>");
			    out.write(((RDFFipaMessage)model).get_reply_by());
			    out.write("</fipa:reply-by>");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_conversation_id().equals("")) ) {
			    out.write("<fipa:conversation-id>");
			    out.write(((RDFFipaMessage)model).get_conversation_id());
			    out.write("</fipa:conversation-id>");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_protocol().equals("")) ) {
			    out.write("<fipa:protocol>");
			    out.write(((RDFFipaMessage)model).get_protocol());
			    out.write("</fipa:protocol>");
			}
		    }
		    catch (Exception f) {}
		     out.write("</fipa:" + ((RDFFipaMessage)model).get_performative() +">\n"); 
	}
	catch (Exception e) {}
	
	out.write("</aktbus:hasContent>");
    }

    public void prettyWriteMessage(Model model, Writer out) {
		try {

	out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?> \n");
	out.write("<rdf:RDF xmlns:rdfs=\"" +RDFS.getURI() + "\" \n");
	out.write("\txmlns:rdf=\"" + RDF.getURI() + "\" \n");
	out.write("\txmlns:aktbus=\"" +  AKTVOCAB.getURI()  + "\"\n");
	out.write("\txmlns:fipa=\"" + FIPAVOCAB.getURI() + "\">\n");

	ResIterator iter = model.listSubjectsWithProperty(RDF.type, AKTVOCAB.Container);
	Resource container = iter.next();
	iter.close();
	Resource header = container.getProperty(AKTVOCAB.hasHeader).getResource();
	Resource content = container.getProperty(AKTVOCAB.hasContent).getResource();

	out.write("<aktbus:Container");
	if (container.hasProperty(AKTVOCAB.id)) {
	out.write(" aktbus:id=");
	out.write("\"" +container.getProperty(AKTVOCAB.id).getString() +"\""); }
	out.write(">\n");

	prettyWriteHeader(header, out);
	prettyWriteContent(model, out);

	out.write("</aktbus:Container>\n");
	out.write("</rdf:RDF>\n");
	
	}
	catch (IOException e)
		{
		System.err.println("Error in I/O...");
		System.err.println(e.toString());
		}
	 catch (Exception e)
		{
		System.err.println("***ERROR: "+e.toString());
		}
    }
  

    public void prettyWriteContent(Model model, Writer out) throws IOException {

	out.write("  <aktbus:hasContent>\n");

	try {
	    ((RDFFipaMessage)model).get_performative();
	    out.write("    <fipa:" + ((RDFFipaMessage)model).get_performative());	   	   

	    try {
		Resource content = ((RDFFipaMessage)model).get_content();
		content.getProperty(FIPAVOCAB.id);
		out.write(" fipa:id=");
		out.write("\"" + content.getProperty(FIPAVOCAB.id).getString() +"\""); 
	    }
	    catch (Exception f) {}

	    out.write(">\n");

	    	    try {
			if (! (((RDFFipaMessage)model).get_fipa_sender().equals("")) ) {
			    out.write("      <fipa:sender>");
			    out.write(((RDFFipaMessage)model).get_fipa_sender());
			    out.write("</fipa:sender>\n");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_fipa_receiver().equals("")) ) {
			    out.write("      <fipa:receiver>");
			    out.write(((RDFFipaMessage)model).get_fipa_receiver());
			    out.write("</fipa:receiver>\n");
			}
		    }
		    catch (Exception f) {}
		    
		    try {
			if (! (((RDFFipaMessage)model).get_fipa_content().equals("")) ) {
			    out.write("      <fipa:content rdf:parseType=\"Literal\">");			   
			    out.write(((RDFFipaMessage)model).get_fipa_content());
			    out.write("</fipa:content>\n");
			}
		    }
		    catch (Exception f) {}
	     
		    try {
			if (! (((RDFFipaMessage)model).get_reply_to().equals("")) ) {
			    out.write("      <fipa:reply-to>");
			    out.write(((RDFFipaMessage)model).get_reply_to());
			    out.write("</fipa:reply-to>\n");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_in_reply_to().equals("")) ) {
			    out.write("      <fipa:in-reply-to>");
			    out.write(((RDFFipaMessage)model).get_in_reply_to());
			    out.write("</fipa:in-reply-to>\n");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_language().equals("")) ) {
			    out.write("      <fipa:language>");
			    out.write(((RDFFipaMessage)model).get_language());
			    out.write("</fipa:language>\n");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_encoding().equals("")) ) {
			    out.write("      <fipa:encoding>");
			    out.write(((RDFFipaMessage)model).get_encoding());
			    out.write("</fipa:encoding>\n");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_ontology().equals("")) ) {
			    out.write("      <fipa:ontology>");
			    out.write(((RDFFipaMessage)model).get_ontology());
			    out.write("</fipa:ontology>\n");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_reply_with().equals("")) ) {
			    out.write("      <fipa:reply-with>");
			    out.write(((RDFFipaMessage)model).get_reply_with());
			    out.write("</fipa:reply-with>\n");
			}
		    }
		    catch (Exception f) {}
	    
		    try {
			if (! (((RDFFipaMessage)model).get_reply_by().equals("")) ) {
			    out.write("      <fipa:reply-by>");
			    out.write(((RDFFipaMessage)model).get_reply_by());
			    out.write("</fipa:reply-by>\n");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_conversation_id().equals("")) ) {
			    out.write("      <fipa:conversation-id>");
			    out.write(((RDFFipaMessage)model).get_conversation_id());
			    out.write("</fipa:conversation-id>\n");
			}
		    }
		    catch (Exception f) {}

		    try {
			if (! (((RDFFipaMessage)model).get_protocol().equals("")) ) {
			    out.write("      <fipa:protocol>");
			    out.write(((RDFFipaMessage)model).get_protocol());
			    out.write("</fipa:protocol>\n");
			}
		    }
		    catch (Exception f) {}
		    out.write("    </fipa:" + ((RDFFipaMessage)model).get_performative() +">\n"); 	 
	}
	catch (Exception e) {}

	out.write("  </aktbus:hasContent>\n");
    }

}
